@@ -373,7 +373,7 @@ static void zval_from_stack_add_frame_parameters(zval *frame, function_stack_ent
 			if (fse->var[j].name && !variadic_opened) {
 				if (Z_TYPE(fse->var[j].data) == IS_UNDEF) {
 					add_assoc_null_ex(params, ZSTR_VAL(fse->var[j].name), ZSTR_LEN(fse->var[j].name));
-				} else {
+				} else if (Z_TYPE(fse->var[j].data) != IS_RESOURCE) {
 					Z_TRY_ADDREF(fse->var[j].data);
 					add_assoc_zval_ex(params, ZSTR_VAL(fse->var[j].name), ZSTR_LEN(fse->var[j].name), &fse->var[j].data);
 				}
@@ -383,11 +383,10 @@ static void zval_from_stack_add_frame_parameters(zval *frame, function_stack_ent
 			/* Unnamed or Variadic parameters */
 			if (Z_TYPE(fse->var[j].data) == IS_UNDEF) {
 				add_index_null(params, j - variadic_opened);
-			} else {
+			} else if (Z_TYPE(fse->var[j].data) != IS_RESOURCE) {
 				Z_TRY_ADDREF(fse->var[j].data);
 				add_index_zval(params, j - variadic_opened, &fse->var[j].data);
 			}
-
 			continue;
 		} else {
 			xdebug_str *argument = NULL;
@@ -463,7 +462,7 @@ static void zval_from_stack_add_frame(zval *output, function_stack_entry *fse, z
 
 	zval_from_stack_add_frame_parameters(frame, fse, params_as_values);
 
-	if (add_local_vars && fse->op_array && fse->op_array->vars) {
+	if (add_local_vars && fse->op_array && fse->op_array->type == ZEND_USER_FUNCTION && fse->op_array->vars && !(fse->function.type & XFUNC_INCLUDES)) {
 		zval_from_stack_add_frame_variables(frame, edata, fse->symbol_table, fse->op_array);
 	}
 
@@ -475,7 +474,7 @@ static void zval_from_stack_add_frame(zval *output, function_stack_entry *fse, z
 	efree(frame);
 }
 
-static void zval_from_stack(zval *output, bool add_local_vars, bool params_as_values)
+static bool zval_from_stack(zval *output, bool add_local_vars, bool params_as_values)
 {
 	function_stack_entry *fse, *next_fse;
 	unsigned int          i;
@@ -483,11 +482,19 @@ static void zval_from_stack(zval *output, bool add_local_vars, bool params_as_va
 	array_init(output);
 
 	fse = XDEBUG_VECTOR_HEAD(XG_BASE(stack));
+
+	/* In some situations, it is possible that there is no stack yet. This
+	 * happens in some cases when Datadog's trace is used. */
+	if (!fse) {
+		return false;
+	}
 	next_fse = fse + 1;
 
 	for (i = 0; i < XDEBUG_VECTOR_COUNT(XG_BASE(stack)) - 1; i++, fse++, next_fse++) {
 		zval_from_stack_add_frame(output, fse, next_fse->execute_data, add_local_vars, params_as_values);
 	}
+
+	return true;
 }
 
 /* Helpers for last_exception_trace slots */
@@ -752,16 +759,14 @@ void xdebug_append_printable_stack(xdebug_str *str, int html)
 				xdebug_str_add_literal(str, "...");
 			}
 
-			if (fse->var[j].name) {
-				if (html) {
-					xdebug_str_add_literal(str, "<span>$");
-					xdebug_str_add_zstr(str, fse->var[j].name);
-					xdebug_str_add_literal(str, " = </span>");
-				} else {
-					xdebug_str_add_literal(str, "$");
-					xdebug_str_add_zstr(str, fse->var[j].name);
-					xdebug_str_add_literal(str, " = ");
-				}
+			if (fse->user_defined == XDEBUG_BUILT_IN && (fse->op_array->fn_flags & ZEND_ACC_USER_ARG_INFO) && fse->op_array->arg_info[j].name) {
+				xdebug_str_add_const(str, html ? "<span>$" : "$");
+				xdebug_str_add_zstr(str, fse->op_array->arg_info[j].name);
+				xdebug_str_add_const(str, html ? " = </span>" : " = ");
+			} else if (fse->var[j].name) {
+				xdebug_str_add_const(str, html ? "<span>$" : "$");
+				xdebug_str_add_zstr(str, fse->var[j].name);
+				xdebug_str_add_const(str, html ? " = </span>" : " = ");
 			}
 
 			if (!variadic_opened && fse->var[j].is_variadic && Z_ISUNDEF(fse->var[j].data)) {
@@ -1241,8 +1246,9 @@ void xdebug_develop_throw_exception_hook(zend_object *exception, zval *file, zva
 	/* Remember last stack trace so it can be retrieved in an exception handler through
 	 * xdebug_get_function_stack(['from_exception' => $e]) */
 	z_last_exception_slot = last_exception_get_slot(exception);
-	zval_from_stack(z_last_exception_slot, true, true);
-	zval_from_stack_add_frame(z_last_exception_slot, XDEBUG_VECTOR_TAIL(XG_BASE(stack)), EG(current_execute_data), true, true);
+	if (zval_from_stack(z_last_exception_slot, true, true)) {
+		zval_from_stack_add_frame(z_last_exception_slot, XDEBUG_VECTOR_TAIL(XG_BASE(stack)), EG(current_execute_data), true, true);
+	}
 
 	exception_trace = tmp_str.d;
 
@@ -1307,9 +1313,11 @@ PHP_FUNCTION(xdebug_get_function_stack)
 
 		value = zend_hash_str_find(options, "from_exception", sizeof("from_exception") - 1);
 		if (value && Z_TYPE_P(value) == IS_OBJECT && instanceof_function(Z_OBJCE_P(value), zend_ce_throwable)) {
-			if (Z_OBJ_P(value) == XG_DEV(last_exception_trace).obj_ptr[0]) {
-				Z_TRY_ADDREF(XG_DEV(last_exception_trace).stack_trace[0]);
-				ZVAL_COPY_VALUE(return_value, &XG_DEV(last_exception_trace).stack_trace[0]);
+			zval *z_previous_exception = last_exception_find_trace(Z_OBJ_P(value));
+
+			if (z_previous_exception) {
+				Z_TRY_ADDREF(*z_previous_exception);
+				ZVAL_COPY_VALUE(return_value, z_previous_exception);
 			} else {
 				array_init(return_value);
 			}
